<?php


namespace Drupal\user_history\Form;


use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Implements the Batch Install Form.
 */
class UserHistoryInitialiseForm extends FormBase {

  /**
   * {@inheritdoc}.
   */
  public function getFormId() {
    return 'user_history_batch_install';
  }

  /**
   * {@inheritdoc}.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $message = 'This process will generate the initial values of the user history records from the user entities in the database.';
    $message .= "\n" . 'There are currently %count user records to be processed.';
    $count_users = count(\Drupal::entityQuery('user')
      ->condition('uid', 0, '>')
      ->execute());

    $form['notice'] = [
      '#type' => 'markup',
      '#value' => $this->t($message, ['%count' => $count_users]),
      '#prefix' => '<div>',
      '#suffix' => '</div>',
    ];

    $form['batch_size'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Batch size'),
      '#description' => $this->t('Enter the number of user accounts to process in each batch'),
      '#default_value' => 100,
      '#required' => TRUE,
    ];

    $form['submit_button'] = [
      '#type' => 'submit',
      '#value' => $this->t('Start Batch'),
    ];

    return $form;
  }


  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $batch_size = (int) $form_state->getValue('batch_size');
    if (!is_integer($batch_size) || $batch_size < 1 || $batch_size > 1000) {
      $form_state->setErrorByName('batch_size', $this->t('Invalid batch size. Specify an integer between 1 and 1000.'));
    }
  }


  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $batch_size = $form_state->getValue('batch_size');

    $module_path = \Drupal::service('module_handler')->getModule('user_history')->getPath();

    $batch = [
      'title' => t('Installing initial user history records...'),
      'operations' => [],
      'init_message' => t('Commencing'),
      'progress_message' => t('Processed @current out of @total.'),
      'error_message' => t('An error occurred during processing'),
      'finished' => 'user_history_finished_initial_history',
      'file' => $module_path . '/user_history.batch.inc',
    ];

    $batch['operations'][] = ['user_history_create_initial_history', [$batch_size]];

    batch_set($batch);

  }

}
