<?php

use Drupal\user_history\Entity\UserHistory;

function user_history_create_initial_history($batch_size, &$context) {

  $entity_type = 'user_history';
  $values = [];
  $current_user = \Drupal::currentUser();

  // Initialise the sandbox values on the first call.
  if (empty($context['sandbox'])) {
    $context['sandbox'] = [];
    $context['sandbox']['total'] = count(\Drupal::entityQuery('user')
      ->condition('uid', 0, '>')
      ->execute());
    $context['sandbox']['last'] = 0;
    $context['sandbox']['count'] = 0;
    // Get current time for entitiy labels
    $now = new \Drupal\Core\Datetime\DrupalDateTime('now', drupal_get_user_timezone());
    $context['sandbox']['date'] = $now->format('Ymd-Hi');
  }

  // Retrieve the next set of user accounts
  $uids = \Drupal::entityQuery('user')
    ->condition('uid', $context['sandbox']['last'], '>')
    ->range(0, $batch_size)
    ->execute();

  $accounts = \Drupal::entityTypeManager()->getStorage('user')->loadMultiple($uids);

  /** @var \Drupal\user\Entity\User $account */
  foreach ($accounts as $account) {
    // Ignore the anonymous account.
    if (!$account->isAnonymous()) {

      $user_history = new UserHistory($values, $entity_type);

      // Set the label for this user_history entity
      $user_history->setLabel($account->id() . ': Install - ' . $context['sandbox']['date']);

      // Set the modified time to the last changed time of the account
      $user_history->setCreatedTime($account->getChangedTime());
      // Set the modified_by uid value to current user.
      $user_history->setModifiedBy($current_user);
      // Record that this record was generated during module install.
      $user_history->setAction('Install');
      // Record that this user entity exists in the database.
      $user_history->setUserDeleted(FALSE);

      $user_history->setUserId($account->id());
      $user_history->setUsername($account->getUsername());
      $user_history->setUserPass($account->getPassword());
      $user_history->setUserMail($account->getEmail());
      $user_history->setUserTimezone($account->getTimeZone());
      $user_history->setUserStatus($account->isActive());
      $user_history->setUserRoles(implode('; ', $account->getRoles(TRUE)));
      $user_history->setUserCreated($account->getCreatedTime());
      $user_history->setUserChanged($account->getChangedTime());
      $user_history->setUserAccess($account->getLastAccessedTime());
      $user_history->setUserLogin($account->getLastLoginTime());
      $user_history->setUserInit($account->getInitialEmail());
      $user_history->setUserLangcode($account->language());
      $user_history->setUserPreferredLangcode($account->getPreferredLangcode());
      $user_history->setUserPreferredAdminLangcode($account->getPreferredAdminLangcode());

      // Mark this user_history as a new entity.
      $user_history->enforceIsNew(TRUE);

      $user_history->save();

    }

    $context['sandbox']['last'] = $account->id();
    $context['sandbox']['count']++;
  }

  if ($context['sandbox']['count'] < $context['sandbox']['total']) {
    $context['finished'] = $context['sandbox']['count'] / $context['sandbox']['total'];
    if ($context['finished'] > 0.99) {
      $context['finished'] = 0.99;
    }
    $context['message'] = t('Processed %count of %total user accounts.', [
      '%count' => $context['sandbox']['count'],
      '%total' => $context['sandbox']['total'],
    ]);
  }
  else {
    $context['finished'] = 1;
    $context['message'] = t('Finished processing %total user accounts.', [
      '%total' => $context['sandbox']['total'],
    ]);
  }

}

function user_history_finished_initial_history($success, $results, $operations) {

  $messenger = \Drupal::messenger();

  if ($success) {
    // Flag the initialisation as complete.
    \Drupal::state()->set('user_history.initialise_required',FALSE);
    // Here we could do something meaningful with the results.
  }
  else {
    // An error occurred.
    // $operations contains the operations that remained unprocessed.
    $error_operation = reset($operations);
    $messenger
      ->addMessage(t('An error occurred while processing @operation with arguments : @args', [
        '@operation' => $error_operation[0],
        '@args' => print_r($error_operation[0], TRUE),
      ]));
  }

}
